-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.relevantDevice = "differential"

local transfercase = nil

--local origViscousCoef = 0
local inputAVSmoother = newExponentialSmoothing(50)
local outputAV1Smoother = newExponentialSmoothing(50)
local outputAV2Smoother = newExponentialSmoothing(50)

--local clutchRatioSmoother = newTemporalSmoothingNonLinear(20, 5)

local function updateGFX(dt)
  if not transfercase then return end
  
  local inputAV = inputAVSmoother:get(transfercase.inputAV)
  local outputAV1 = outputAV1Smoother:get(transfercase.outputAV1) 
  local outputAV2 = outputAV2Smoother:get(transfercase.outputAV2) 
  local throttle = electrics.values['throttle_input']
  local brake = electrics.values['brake_input']
  local steer = electrics.values['steering_input']
  local absSteer = math.abs(steer)
  local horizontalG = sensors.gy2
  local absHorizontalG = math.abs(horizontalG) --negative is forward
  local lateralG = sensors.gx2
  local absLateralG = math.abs(lateralG)
  local steerDirectionCoef = clamp(steer * 10,-1,1)
  local sideAccelDirectionCoef = clamp(- lateralG * 1,-1,1)


  if electrics.values['cDiffMode'] == 1 then --Powerslide/Rally
    local tightCorneringCtrl = math.pow(math.min(math.abs(lateralG/11 + steer), 1.0), 1.0)
    --transfercase.lsdLockCoef = 0.1 + math.max(throttle, brake) * 0.4 * (1.0 - math.pow(absSteer, 1))
    transfercase.lsdLockCoef = 0.1 + math.max((0.3 - 0.3 * tightCorneringCtrl),0)
    --transfercase.lsdPreload = 50
    transfercase.lsdPreload = 8 + math.max(throttle, brake * 0.5) * 75 * (1.0 - math.pow(absSteer, 0.7)) + clamp(math.abs(outputAV2 - outputAV1) * 0.004,0,1) * 75
  elseif electrics.values['cDiffMode'] == 2 then --Passive/Snow
    transfercase.lsdLockCoef = 0.1 
    transfercase.lsdPreload = 1 + clamp(math.abs(outputAV2 - outputAV1) * 0.004,0,1) * 500
  elseif electrics.values['cDiffMode'] == 0 then --Performance/Tarmac
    local tightCorneringCtrl = math.pow(math.min(math.abs(lateralG/11 + steer), 1.0), 1.0)
    transfercase.lsdLockCoef = math.max((0.5 - 0.5 * tightCorneringCtrl),0)
    transfercase.lsdPreload = 10 + math.max(throttle, brake) * 200 * (1.0 - math.pow(absSteer, 1))
  else --Offroad
    transfercase.lsdLockCoef = 0.5
    transfercase.lsdPreload = 1
  end
  
  if input.parkingbrake > 0.2 then
    transfercase.lsdLockCoef = 0
    transfercase.lsdPreload = 1
  end  
    
	transfercase.lsdRevLockCoef = transfercase.lsdLockCoef

end


local function setDiffMode()
	if electrics.values['cDiffMode'] > 3 then 
	electrics.values['cDiffMode'] = 0
	end
	if electrics.values['cDiffMode'] < 0 then 
	electrics.values['cDiffMode'] = 3
	end

	if electrics.values['cDiffMode'] == 0 then 
	  guihooks.message({txt = "C.Diff Tarmac"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffMode'] == 1 then 
	  guihooks.message({txt = "C.Diff Gravel"}, 3, "vehicle.cDiffMode")
	end 
	if electrics.values['cDiffMode'] == 2 then 
	  guihooks.message({txt = "C.Diff Snow"}, 3, "vehicle.cDiffMode")
	end

	if electrics.values['cDiffMode'] == 3 then 
	  guihooks.message({txt = "C.Diff Offroad"}, 3, "vehicle.cDiffMode")
	end

end

local function nextDiffMode()
  electrics.values['cDiffMode'] = electrics.values['cDiffMode'] + 1
  setDiffMode()
end

local function previousDiffMode()
  electrics.values['cDiffMode'] = electrics.values['cDiffMode'] - 1
  setDiffMode()
end

local function init(jbeamData)
  transfercase = powertrain.getDevice(jbeamData.transfercaseName)
  if transfercase then
    transfercase.lsdLockCoef = 0.10
	transfercase.lsdRevLockCoef = 0.10
    transfercase.lsdPreload = 10
	electrics.values['cDiffMode'] = 0
	guihooks.message({txt = "C.Diff Tarmac"}, 3, "vehicle.cDiffMode")
  end

end

M.init = init
--M.reset = init
M.updateGFX = updateGFX
M.setDiffMode = setDiffMode
M.nextDiffMode = nextDiffMode
M.previousDiffMode = previousDiffMode

return M