-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.relevantDevice = "differential"

local transfercase = nil

--local origViscousCoef = 0
local inputAVSmoother = newExponentialSmoothing(50)
local outputAV1Smoother = newExponentialSmoothing(50)
local outputAV2Smoother = newExponentialSmoothing(50)

--local clutchRatioSmoother = newTemporalSmoothingNonLinear(20, 5)

local function updateGFX(dt)
  if not transfercase then return end
  
  local inputAV = inputAVSmoother:get(transfercase.inputAV)
  local outputAV1 = outputAV1Smoother:get(transfercase.outputAV1) 
  local outputAV2 = outputAV2Smoother:get(transfercase.outputAV2) 
  local throttle = electrics.values['throttle_input']
  local brake = electrics.values['brake_input']
  local steer = electrics.values['steering_input']
  local absSteer = math.abs(steer)
  local horizontalG = sensors.gy2
  local absHorizontalG = math.abs(horizontalG) --negative is forward
  local lateralG = sensors.gx2
  local absLateralG = math.abs(lateralG)
  local steerDirectionCoef = clamp(steer * 10,-1,1)
  local sideAccelDirectionCoef = clamp(- lateralG * 1,-1,1)

  if electrics.values['cDiffMode'] == 1 then --Manual

    if electrics.values['cDiffModeM'] == 0 then --0%
	  transfercase.lsdLockCoef = 0.0
	  transfercase.lsdPreload = 0
	end
    if electrics.values['cDiffModeM'] == 1 then --20%
	  transfercase.lsdLockCoef = 0.1
	  transfercase.lsdPreload = 10
	end
    if electrics.values['cDiffModeM'] == 2 then --40%
	  transfercase.lsdLockCoef = 0.2
	  transfercase.lsdPreload = 20
	end
    if electrics.values['cDiffModeM'] == 3 then --60%
	  transfercase.lsdLockCoef = 0.3
	  transfercase.lsdPreload = 30
	end
    if electrics.values['cDiffModeM'] == 4 then --80%
	  transfercase.lsdLockCoef = 0.4
	  transfercase.lsdPreload = 40
	end
    if electrics.values['cDiffModeM'] == 5 then --100%
	  transfercase.lsdLockCoef = 0.5
	  transfercase.lsdPreload = 50
	end

  else --Auto
  
    if electrics.values['cDiffModeA'] == 0 then --Auto -
      local tightCorneringCtrl = math.pow(math.min(math.abs(lateralG/11 + steer), 1.0), 1.0)
      transfercase.lsdLockCoef = 0.15
      transfercase.lsdPreload = 8 + math.max(throttle * 0.5, brake * 0.25) * 250 * (1.0 - math.pow(clamp(tightCorneringCtrl * 2,0,1), 1))
    end
    if electrics.values['cDiffModeA'] == 2 then --Auto+
      transfercase.lsdLockCoef = 0.15
      transfercase.lsdPreload = 10 + clamp(0.2 + 0.4 * clamp(throttle * 2 - 0.2,0,1) + 0.4 * clamp(math.abs(outputAV2 - outputAV1) * 0.02 - 0.4,0,1),0,1) * 400
    end
    if electrics.values['cDiffModeA'] == 1 then --Auto
      local tightCorneringCtrl = math.pow(math.min(math.abs(lateralG/11 + steer), 1.0), 1.0)
      transfercase.lsdLockCoef = 0.15
      transfercase.lsdPreload = 8 + math.max(throttle, brake) * 200 * (1.0 - math.pow(tightCorneringCtrl, 1))
    end

  end
  
  if input.parkingbrake > 0.2 then
    transfercase.lsdLockCoef = 0
    transfercase.lsdPreload = 1
  end  
    
	transfercase.lsdRevLockCoef = transfercase.lsdLockCoef

end


local function setDiffMode()
  if electrics.values['cDiffMode'] == 0 then --Auto
	if electrics.values['cDiffModeA'] > 2 then 
	electrics.values['cDiffModeA'] = 2
	end
	if electrics.values['cDiffModeA'] < 0 then 
	electrics.values['cDiffModeA'] = 0
	end

	if electrics.values['cDiffModeA'] == 0 then 
	  guihooks.message({txt = "C.Diff Auto -"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffModeA'] == 1 then 
	  guihooks.message({txt = "C.Diff Auto"}, 3, "vehicle.cDiffMode")
	end 
	if electrics.values['cDiffModeA'] == 2 then 
	  guihooks.message({txt = "C.Diff Auto +"}, 3, "vehicle.cDiffMode")
	end

  else --Manual
	if electrics.values['cDiffModeM'] > 5 then 
	electrics.values['cDiffModeM'] = 5
	end
	if electrics.values['cDiffModeM'] < 0 then 
	electrics.values['cDiffModeM'] = 0
	end

	if electrics.values['cDiffModeM'] == 0 then 
	  guihooks.message({txt = "C.Diff 0%"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffModeM'] == 1 then 
	  guihooks.message({txt = "C.Diff 20%"}, 3, "vehicle.cDiffMode")
	end 
	if electrics.values['cDiffModeM'] == 2 then 
	  guihooks.message({txt = "C.Diff 40%"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffModeM'] == 3 then 
	  guihooks.message({txt = "C.Diff 60%"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffModeM'] == 4 then 
	  guihooks.message({txt = "C.Diff 80%"}, 3, "vehicle.cDiffMode")
	end
	if electrics.values['cDiffModeM'] == 5 then 
	  guihooks.message({txt = "C.Diff Locked"}, 3, "vehicle.cDiffMode")
	end
  end
end

local function toggleDiffMode()
  if electrics.values['cDiffMode'] == 0 then
    electrics.values['cDiffMode'] = 1
  else
    electrics.values['cDiffMode'] = 0
  end
  setDiffMode()
end

local function nextDiffMode()
  if electrics.values['cDiffMode'] == 0 then
    electrics.values['cDiffModeA'] = electrics.values['cDiffModeA'] + 1
  else
    electrics.values['cDiffModeM'] = electrics.values['cDiffModeM'] + 1
  end
  setDiffMode()
end

local function previousDiffMode()
  if electrics.values['cDiffMode'] == 0 then
    electrics.values['cDiffModeA'] = electrics.values['cDiffModeA'] - 1
  else
    electrics.values['cDiffModeM'] = electrics.values['cDiffModeM'] - 1
  end
  setDiffMode()
end

local function init(jbeamData)
  transfercase = powertrain.getDevice(jbeamData.transfercaseName)
  if transfercase then
    transfercase.lsdLockCoef = 0.15
	transfercase.lsdRevLockCoef = 0.15
	electrics.values['cDiffMode'] = 0
	electrics.values['cDiffModeA'] = 1
	electrics.values['cDiffModeM'] = 1
	guihooks.message({txt = "C.Diff Auto"}, 3, "vehicle.cDiffMode")
  end

end

M.init = init
--M.reset = init
M.updateGFX = updateGFX
M.setDiffMode = setDiffMode
M.toggleDiffMode = toggleDiffMode
M.nextDiffMode = nextDiffMode
M.previousDiffMode = previousDiffMode

return M