-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxilliary"
M.relevantDevice = "differential_R"
M.inputValues = nil
local differential = nil

local inputAVSmoother = newExponentialSmoothing(50)
local outputAV1Smoother = newExponentialSmoothing(50)
local outputAV2Smoother = newExponentialSmoothing(50)
--local inputTorqueSmoother = newExponentialSmoothing(50)
local outputTorque1Smoother = newExponentialSmoothing(50)
local outputTorque2Smoother = newExponentialSmoothing(50)

local function updateGFX()

  --local inputTorque = inputTorqueSmoother:get(differential.inputTorque)
  local outputTorque1 = outputTorque1Smoother:get(differential.outputTorque1) --left wheel
  local outputTorque2 = outputTorque2Smoother:get(differential.outputTorque2) --right wheel
  local inputAV = inputAVSmoother:get(differential.inputAV)
  local outputAV1 = outputAV1Smoother:get(differential.outputAV1) --left wheel
  local outputAV2 = outputAV2Smoother:get(differential.outputAV2) --right wheel
  local throttle = electrics.values['throttle_input']
  local brake = electrics.values['brake_input']
  local steer = electrics.values['steering_input']  --positive is right turn
  local absSteer = math.abs(steer)
  local lateralG = sensors.gx2  --negative is right turn
  local absLateralG = math.abs(lateralG)
  local steerDirectionCoef = clamp(steer * 10,-1,1) --positive is right turn
  local sideAccelDirectionCoef = clamp(- lateralG * 1,-1,1) --positive is right turn

  local powersign = clamp(inputAV * (outputTorque1 + outputTorque2),-1,1) --negative is engine braking
  local steersign = clamp(steerDirectionCoef * sideAccelDirectionCoef * 10,-1,1) -- negative is countersteer
  local steeringlock = clamp(400 / (math.pow(math.min(math.abs(outputAV2),math.abs(outputAV1)),2) + 1e-30),0.05,1) --steering required to reach maximum side acceleration 
  local steeringtvc = clamp(steer / steeringlock,-1,1) --proactive torque bias handling enhancement

  local weighttransfertvc = clamp(-lateralG/11,-1,1)

  --differential.lsdLockCoef = 0.02 + 0.18 * clamp(throttle * 10,0,1)
  differential.lsdLockCoef = 0.01 + 0.09 * clamp((outputAV2 - outputAV1) * sideAccelDirectionCoef * 2 * math.max(steersign,0) * clamp(throttle * 5 - 1,0,1),0,1) --reactive torque bias for reducing inner wheelspin
  --differential.lsdLockCoef = 0.02

  differential.diffTorqueSplitA = 0.5 + clamp(0.1 * weighttransfertvc + 0.25 * steeringtvc * math.max(powersign,0),-0.3,0.3) --torque vectoring (proactive torque bias)
  --differential.diffTorqueSplitA = clamp(((steer - lateralG/11) * 0.5 * clamp(throttle * 5,0,1) + 0.5), 0.25, 0.75)
  --differential.diffTorqueSplitA = 0.5  --left wheel

  --differential.lsdPreload = 5 + 2500 * clamp(math.abs(outputAV2 - outputAV1)*0.5,0,1)
  --differential.lsdPreload = 5 + 250 * steeringlock * clamp(math.abs(outputAV2 - outputAV1) * 0.2 - 1.0,0,1) * clamp((throttle - 0.1) * 10,0.1,1) --reactive locking for getting unstuck
  differential.lsdPreload = 5 + 200 * clamp(math.pow(steeringlock,2) + math.max(-powersign,0),0,1) * clamp(math.abs(outputAV2 - outputAV1) * 0.2 - 1.0,0,1) + 2000 * math.max(-powersign,0) * math.max(-steersign,0) --reactive locking for getting unstuck
  --differential.lsdPreload = 0
  
  differential.diffTorqueSplitB = 1 - differential.diffTorqueSplitA --right wheel
  differential.lsdRevLockCoef = differential.lsdLockCoef
  
  --log('I', "Steeringlock ", steeringlock)
  --log('I', "Steeringtvc ", steeringtvc)
  --log('I', "anti-spin ", math.max(-powersign,0) * math.max(-steersign,0))
  
end

local function init(jbeamData)
  differential = powertrain.getDevice(jbeamData.transfercaseName)

  if differential then
    differential.lsdLockCoef = 0.1
    differential.lsdRevLockCoef = 0.1
    differential.lsdPreLoad = 10
    differential.diffTorqueSplit = 0.5

    M.updateGFX = updateGFX
  end
end

M.init = init
M.reset = nop
M.updateGFX = nop

return M